<?php

namespace Drupal\hidden_tab;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Manipulates entity type information.
 *
 * This class contains primarily bridged hooks for compile-time or
 * cache-clear-time hooks. Runtime hooks should be placed in EntityOperations.
 */
class EntityTypeInfo implements ContainerInjectionInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  public function __construct(EntityTypeManagerInterface $entityTypeManager, AccountInterface $current_user) {
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('current_user')
    );
  }

  public function entityOperation(EntityInterface $entity) {
    if ($entity->getEntityTypeId() !== 'node' || !$this->currentUser->hasPermission(Utility::ADMIN_PERMISSION)) {
      return [];
    }

    $pages = $this->entityTypeManager
      ->getStorage('hidden_tab_page')
      ->loadMultiple();

    $operations = [];
    foreach ($pages as $page_id => $page) {
      $operations['hidden_tab_tab_' . $page_id] = [
        'title' => $page->label(),
        'weight' => 100,
        'url' => Url::fromRoute('hidden_tab.tab_' . $page_id, [
          'node' => $entity->id(),
        ]),
      ];
      $operations['hidden_tab_uri_' . $page_id] = [
        'title' => $page->label(),
        'weight' => 100,
        'url' => Url::fromRoute('hidden_tab.uri_' . $page_id, [
          'node' => $entity->id(),
        ]),
      ];
    }
    return $operations;
  }

}
