<?php

namespace Drupal\hidden_tab\Form;

use Drupal\Component\Uuid\Php;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\hidden_tab\Entity\HiddenTabCredit;
use Drupal\hidden_tab\Entity\HiddenTabPageInterface;
use Drupal\hidden_tab\Form\Base\OnPageAdd;
use Drupal\hidden_tab\Service\CreditChargingInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * To add credit directly on tab page.
 *
 * @see \Drupal\hidden_tab\Entity\HiddenTabCreditInterface
 * @see \Drupal\hidden_tab\Controller\XPageRenderController
 * @property \Drupal\hidden_tab\Entity\HiddenTabCreditInterface $entity
 *
 */
class OnPageAddCreditForm extends OnPageAdd {

  /**
   * {@inheritdoc}
   */
  protected $ID = 'hidden_tab_on_page_add_credit_form';

  /**
   * {@inheritdoc}
   */
  protected $prefix = 'hidden_tab_on_page_add_credit_form_';

  /**
   * {@inheritdoc}
   */
  protected $label = 'Credit';

  /**
   * {@inheritdoc}
   */
  protected static $type = 'hidden_tab_credit';

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * @var \Drupal\hidden_tab\Service\CreditChargingInterface
   */
  protected $creditCharging;

  public function __construct(EntityStorageInterface $entity_storage,
                              Php $uuid,
                              EntityTypeManagerInterface $entity_type_manager,
                              CreditChargingInterface $credit_charging) {
    parent::__construct($entity_storage, $uuid);
    $this->entityTypeManager = $entity_type_manager;
    $this->creditCharging = $credit_charging;
  }


  public static function create(ContainerInterface $container) {
    /** @noinspection PhpParamsInspection */
    return new static(
      $container->get('entity_type.manager')->getStorage(static::$type),
      $container->get('uuid'),
      $container->get('entity_type.manager'),
      $container->get('hidden_tab.credit_service')
    );
  }


  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    HiddenTabCredit::validateForm(
      $this->entityTypeManager,
      $this->creditCharging,
      $form_state,
      $this->prefix,
      TRUE,
      'node',
      NULL
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getFormElements(EntityInterface $target_entity, HiddenTabPageInterface $page): array {
    return HiddenTabCredit::littleForm($this->uuid);
  }

  /**
   * {@inheritdoc}
   */
  protected function getValues(FormStateInterface $form_state): array {
    $form = HiddenTabCredit::littleForm($this->uuid, $this->prefix, TRUE);
    foreach ([
               'target_entity',
               'target_entity_type',
               'target_entity_bundle',
             ] as $item) {
      unset($form[$this->prefix . $item]);
    }
    return $form;
  }

}
