<?php

namespace Drupal\hidden_tab\Plugin\HiddenTabRender;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Logger\LoggerChannel;
use Drupal\Core\Session\AccountInterface;
use Drupal\hidden_tab\Entity\HiddenTabPageInterface;
use Drupal\hidden_tab\Form\OnPageAddCreditForm;
use Drupal\hidden_tab\Plugable\Annotation\HiddenTabRenderAnon;
use Drupal\hidden_tab\Plugable\Render\HiddenTabRenderAdministrativeBase;
use Drupal\hidden_tab\Plugable\Render\HiddenTabRenderSafeTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\ParameterBag;
use Throwable;
use function str_replace;

/**
 * Displays an add credit form on a hidden tab page.
 *
 * @HiddenTabRenderAnon(
 *   id = "hidden_tab_add_credit"
 * )
 */
class HiddenTabRenderAddCredit extends HiddenTabRenderAdministrativeBase {

  use HiddenTabRenderSafeTrait;

  /**
   * See id().
   *
   * @var string
   *
   * @see \Drupal\hidden_tab\Plugable\HiddenTabPluginInterfaceBase::id()
   */
  protected $PID = 'hidden_tab_add_credit';

  /**
   * See label().
   *
   * @var string
   *
   * @see \Drupal\hidden_tab\Plugable\HiddenTabPluginInterfaceBase::label()
   */
  protected $HTPLabel = 'Add Credit';

  /**
   * See description().
   *
   * @var string
   *
   * @see \Drupal\hidden_tab\Plugable\HiddenTabPluginInterfaceBase::description()
   */
  protected $HTPDescription = 'Displays an add credit form on a hidden tab page.';

  /**
   * See weight().
   *
   * @var string
   *
   * @see \Drupal\hidden_tab\Plugable\HiddenTabPluginInterfaceBase::weight()
   */
  protected $HTPWeight = 4;

  /**
   * See tags().
   *
   * @var string
   *
   * @see \Drupal\hidden_tab\Plugable\HiddenTabPluginInterfaceBase::tags()
   */
  protected $HTPTags = [];

  /**
   * To build the add form.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * @var \Drupal\Core\Logger\LoggerChannel
   */
  protected $logger;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration,
                              string $plugin_id,
                              $plugin_definition,
                              FormBuilderInterface $form_builder,
                              LoggerChannel $logger) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->formBuilder = $form_builder;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container,
                                array $configuration,
                                $plugin_id,
                                $plugin_definition) {
    /** @noinspection PhpParamsInspection */
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('form_builder'),
      $container->get('logger.factory')->get('hidden_tab')
    );
  }

  protected function render0(EntityInterface $entity,
                             HiddenTabPageInterface $page,
                             AccountInterface $user,
                             ParameterBag $bag,
                             array &$output) {
    try {
      $this->render1($entity, $page, $user, $bag, $output);
    }
    catch (Throwable $error) {
      $output[$this->id()] = [
        '#type' => 'markup',
        '#markup' => t('There was an error generating add credit form'),
      ];
      $this->logger->error('HiddenTabRenderAddCredit::render0() error while creating on page add credit form page={ht_page_id} entity={ht_entity_id} entity-type={ht_type} user={ht_user_id} msg={ht_msg} trace=<pre>{ht_trace}</pre>', [
        'ht_page_id' => $page->id(),
        'ht_entity_id' => $entity->id(),
        'ht_type' => $entity->getEntityTypeId(),
        'ht_user_id' => $user->id(),
        'ht_msg' => $error->getMessage(),
        'ht_trace' => $error->getTraceAsString(),
      ]);
    }
  }

  protected function render1(EntityInterface $entity,
                             HiddenTabPageInterface $page,
                             AccountInterface $use,
                             ParameterBag $bag,
                             array &$output) {
    /** @noinspection PhpMethodParametersCountMismatchInspection */
    $form = $this->formBuilder->getForm(OnPageAddCreditForm::class, $entity, $page);
    $output['admin'][$this->id()] = $form;
  }

}
